package com.cj.jiaqingjiayi.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.CommonRequest;
import com.cj.jiaqingjiayi.model.domain.Appointments;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.appointments.AppointmentsAddRequest;
import com.cj.jiaqingjiayi.model.request.appointments.AppointmentsUpdateRequest;
import com.cj.jiaqingjiayi.model.vo.AppointmentsVO;
import com.cj.jiaqingjiayi.service.AppointmentsService;
import com.cj.jiaqingjiayi.service.UserService;
import com.cj.jiaqingjiayi.utils.BeanCopyUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

@Slf4j
@Api(tags = "预约接口")
@RestController
@RequestMapping("/appointments")
public class AppointmentsController {

    @Resource
    private UserService userService;

    @Resource
    private AppointmentsService appointmentsService;

    @ApiOperation(value = "添加预约")
    @PostMapping("/add")
    public BaseResponse<Long> addAppointments(@RequestBody AppointmentsAddRequest appointmentsAddRequest, HttpServletRequest request){
        //校验是否登录
//        User loginUser = userService.getLoginUser(request);
        ThrowUtils.throwIf(appointmentsAddRequest== null, ErrorCode.NULL_ERROR);

        String appointmentTime = appointmentsAddRequest.getAppointmentTime();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("MM-dd HH:mm");
        // 提取当前年份
        int year = LocalDate.now().getYear();
        // 解析成 LocalDate 和 LocalTime
        LocalDate date = LocalDate.parse(year + "-" + appointmentTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm"));
        LocalTime time = LocalTime.parse(appointmentTime, formatter);
        // 合并为 LocalDateTime
        LocalDateTime dateTime = LocalDateTime.of(date, time);
        // LocalDateTime 转为 Date
        Date date1 = Date.from(dateTime.atZone(ZoneId.systemDefault()).toInstant());


        Appointments appointments = new Appointments();
        appointments.setAppointmentTime(date1);
        //不传入默认用户昵称
//        if (appointmentsAddRequest.getUsername() == null) {
//            appointmentsAddRequest.setUsername(loginUser.getUsername());
//        }
        BeanUtils.copyProperties(appointmentsAddRequest, appointments);
        appointments.setUserId(appointments.getUserId());
        //校验是否输入必要的数据
        appointmentsService.valid(appointments);

        //返回预约号
        Long id = appointmentsService.addAppointments(appointments);
        ThrowUtils.throwIf(id < 0, ErrorCode.PARAMS_ERROR, "预约失败");

        return ResultUtils.success(id, "预约成功");
    }

    @ApiOperation(value = "查询我的预约")
    @PostMapping("/query")
    public BaseResponse<List<AppointmentsVO>> queryMyAppointments(@RequestBody CommonRequest commonRequest, HttpServletRequest request){
//        User loginUser = userService.getLoginUser(request);
        Long userid = commonRequest.getId();
        QueryWrapper<Appointments> wrapper = new QueryWrapper<>();
        wrapper.eq("userId", userid);
        List<Appointments> appointmentsList = appointmentsService.list(wrapper);

        List<AppointmentsVO> appointmentsVOList = appointmentsService.queryAppointmentsVO(appointmentsList);
        return ResultUtils.success(appointmentsVOList);
    }

    @PostMapping("/remove")
    @ApiOperation(value = "取消预约")
    public BaseResponse<Boolean> removeAppointments(@RequestBody CommonRequest commonRequest) {

        boolean flag = appointmentsService.removeId(commonRequest.getId());

        return ResultUtils.success(flag);
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改预约信息")
    public BaseResponse<Boolean> updateAppointments(@RequestBody AppointmentsUpdateRequest updateRequest) {
        ThrowUtils.throwIf(updateRequest == null, ErrorCode.NULL_ERROR);


        Long id = updateRequest.getId();
        Appointments appointments = appointmentsService.getById(id);
        BeanCopyUtils.copyPropertiesIgnoreEmpty(updateRequest,appointments);
        boolean flag = appointmentsService.updateById(appointments);

        return ResultUtils.success(flag);
    }

    @GetMapping("/list/select")
    @ApiOperation(value = "查询当前店铺预约信息")
    public BaseResponse<List<Appointments>> queryBuAppointments(@RequestParam Long id) {
        ThrowUtils.throwIf(id < 0, ErrorCode.NULL_ERROR);

        QueryWrapper<Appointments> queryWrapper = new QueryWrapper<>();
        List<Appointments> appointmentsList = appointmentsService.queryAppointments(queryWrapper);

        return ResultUtils.success(appointmentsList);
    }
}
