package com.cj.jiaqingjiayi.utils;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;

import java.beans.PropertyDescriptor;
import java.util.HashSet;
import java.util.Set;

public class BeanCopyUtils {

    /**
     * 拷贝属性，跳过 null 和空字符串
     */
    public static void copyPropertiesIgnoreEmpty(Object source, Object target) {
        BeanUtils.copyProperties(source, target, getNullOrEmptyPropertyNames(source));
    }

    /**
     * 获取属性值为 null 或空字符串的属性名
     */
    private static String[] getNullOrEmptyPropertyNames(Object source) {
        final BeanWrapper src = new BeanWrapperImpl(source);
        PropertyDescriptor[] pds = src.getPropertyDescriptors();

        Set<String> emptyNames = new HashSet<>();
        for (PropertyDescriptor pd : pds) {
            Object value = src.getPropertyValue(pd.getName());
            if (value == null || "".equals(value)) {
                emptyNames.add(pd.getName());
            }
        }
        return emptyNames.toArray(new String[0]);
    }
}
